#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <list>

#include "opengl.h"

#include "SDL.h"
#include "SDL_opengl.h"

static SDL_Surface *global_image = NULL;
static GLuint global_texture = 0;

#include "util.h"
#include "entity.h"
#include "txtprt_import.h"
#include "randlvl.h"

bool doom3parse (const char *f);

#include "shader.h"

/**********************************************************************/

view camera;
vec3 camera_velocity;
float camera_angle, camera_angle2, camera_velocity_angle, camera_velocity_angle2;
float camera_velocity_factor = 80;

void
HotKey_ToggleFullScreen (void)
{
  SDL_Surface *screen;

  screen = SDL_GetVideoSurface ();

  if (SDL_WM_ToggleFullScreen (screen))
    printf ("Toggled fullscreen mode - now %s\n",
            (screen->flags & SDL_FULLSCREEN) ? "fullscreen" : "windowed");
  else
    printf ("Unable to toggle fullscreen mode\n");
}

void
HotKey_ToggleGrab (void)
{
  SDL_GrabMode mode;

  printf ("Ctrl-G: toggling input grab!\n");
  mode = SDL_WM_GrabInput (SDL_GRAB_QUERY);

  if (mode == SDL_GRAB_ON)
    printf ("Grab was on\n");
  else
    printf ("Grab was off\n");

  mode = SDL_WM_GrabInput (mode ? SDL_GRAB_OFF : SDL_GRAB_ON);

  if (mode == SDL_GRAB_ON)
    printf ("Grab is now on\n");
  else
    printf ("Grab is now off\n");
}

void
HotKey_Iconify (void)
{
  printf ("Ctrl-Z: iconifying window!\n");
  SDL_WM_IconifyWindow ();
}

int
HandleEvent (SDL_Event * event)
{
  int done;

  done = 0;

  switch (event->type)
    {
    case SDL_ACTIVEEVENT:
      /* See what happened */
      printf ("app %s ", event->active.gain ? "gained" : "lost");
      if (event->active.state & SDL_APPACTIVE)
        printf ("active ");
      else if (event->active.state & SDL_APPMOUSEFOCUS)
        printf ("mouse ");
      else if (event->active.state & SDL_APPINPUTFOCUS)
        printf ("input ");

      printf ("focus\n");
      break;

    case SDL_KEYDOWN:
      if (event->key.keysym.sym == SDLK_UP)       camera_velocity.z--;
      if (event->key.keysym.sym == SDLK_DOWN)     camera_velocity.z++;
      if (event->key.keysym.sym == SDLK_RIGHT)    camera_velocity_angle++;
      if (event->key.keysym.sym == SDLK_LEFT)     camera_velocity_angle--;
      if (event->key.keysym.sym == SDLK_PAGEUP)   camera_velocity_angle2++;
      if (event->key.keysym.sym == SDLK_PAGEDOWN) camera_velocity_angle2--;
      if (event->key.keysym.sym == SDLK_e)        camera_velocity.y--;
      if (event->key.keysym.sym == SDLK_q)        camera_velocity.y++;
      if (event->key.keysym.sym == SDLK_v)        camera_velocity_factor *= 1.5;
      if (event->key.keysym.sym == SDLK_b)        camera_velocity_factor /= 1.5;

      if (event->key.keysym.sym == SDLK_ESCAPE)
	done = 1;

      if ((event->key.keysym.sym == SDLK_g) &&
	  (event->key.keysym.mod & KMOD_CTRL))
	HotKey_ToggleGrab ();

      if ((event->key.keysym.sym == SDLK_z) &&
	  (event->key.keysym.mod & KMOD_CTRL))
	HotKey_Iconify ();

      if ((event->key.keysym.sym == SDLK_RETURN) &&
	  (event->key.keysym.mod & KMOD_ALT))
	HotKey_ToggleFullScreen ();

      break;

    case SDL_KEYUP:
      if (event->key.keysym.sym == SDLK_UP)    camera_velocity.z++;
      if (event->key.keysym.sym == SDLK_DOWN)  camera_velocity.z--;
      if (event->key.keysym.sym == SDLK_LEFT)  camera_velocity_angle++;
      if (event->key.keysym.sym == SDLK_RIGHT) camera_velocity_angle--;
      if (event->key.keysym.sym == SDLK_PAGEUP)   camera_velocity_angle2--;
      if (event->key.keysym.sym == SDLK_PAGEDOWN) camera_velocity_angle2++;
      if (event->key.keysym.sym == SDLK_e)     camera_velocity.y++;
      if (event->key.keysym.sym == SDLK_q)     camera_velocity.y--;
      break;

    case SDL_QUIT:
      done = 1;
      break;
    }

  return (done);
}

void draw_floor (int size, int dx, int dy, int dz)
{
  int x, z, ry;

  for (x = 0; x < 10; x++)
    {
      for (z = 0; z < 10; z++)
        {
          vector<vertex_t2f_n3f_v3f> pts;
          pts.push_back (vertex_t2f_n3f_v3f (point (   0, 0,    0), vec3 (0, 1, 0), tex2 (0, 0)));
          pts.push_back (vertex_t2f_n3f_v3f (point (   0, 0, size), vec3 (0, 1, 0), tex2 (0, 1)));
          pts.push_back (vertex_t2f_n3f_v3f (point (size, 0, size), vec3 (0, 1, 0), tex2 (1, 1)));
          pts.push_back (vertex_t2f_n3f_v3f (point (size, 0,    0), vec3 (0, 1, 0), tex2 (1, 0)));

          geometry_quads *q = new geometry_quads;
          //q->m = new simple_material;
          q->set (pts);
          entity *e = new entity (q);
          e->move (vec3 (dx + x * size, dy, dz + z * size));
          e->show ();
        }
    }
}

void draw_test_nurb ()
{
  geometry_nurbs *q = new geometry_nurbs;
  q->set ();
  entity *e = new entity (q);
  e->move (vec3 (10, 3, -4));
  e->show ();
}

void fisch (vec3 center, GLfloat radius, int depth)
{
  entity *planet = new entity (new geometry_sphere (testmat, radius));
  planet->move (center);
  planet->show ();

  if (--depth)
    {
      GLfloat r2 = radius * .1F;
      GLfloat r3 = radius + r2;

      fisch (center + vec3 (r3, 0, 0), r2, depth);
      fisch (center + vec3 (0, 0, r3), r2, depth);
      fisch (center + vec3 (-r3, 0, 0), r2, depth);
    }
}

extern void draw_level ();

std::list<entity_moveable *> moveables;

void perfom_moves ()
{
  for (std::list<entity_moveable *>::iterator i = moveables.begin (); i != moveables.end (); ++i)
    {
      (*i)->perform_step (timer.diff);
    }
}

int
RunGLTest (int argc, char *argv[],
	   int logo, int slowly, int bpp, float gamma, int noframe, int fsaa)
{
  int i;
  int rgb_size[3];
  int w = 800;
  int h = 600;
  int done = 0;
  int frames;
  Uint32 video_flags;
  int value;
  GLenum gl_error;

  if (SDL_Init (SDL_INIT_VIDEO) < 0)
    {
      fprintf (stderr, "Couldn't initialize SDL: %s\n", SDL_GetError ());
      exit (1);
    }

  /* See if we should detect the display depth */
  if (bpp == 0)
    {
      if (SDL_GetVideoInfo ()->vfmt->BitsPerPixel <= 8)
	bpp = 8;
      else
	bpp = 16;		/* More doesn't seem to work */
    }

  video_flags = SDL_OPENGL;

  for (i = 1; argv[i]; ++i)
    if (strcmp (argv[1], "-fullscreen") == 0)
      video_flags |= SDL_FULLSCREEN;

  if (noframe)
    video_flags |= SDL_NOFRAME;

  /* Initialize the display */
  switch (bpp)
    {
    case 8:
      rgb_size[0] = 3;
      rgb_size[1] = 3;
      rgb_size[2] = 2;
      break;

    case 15:
    case 16:
      rgb_size[0] = 5;
      rgb_size[1] = 5;
      rgb_size[2] = 5;
      break;

    default:
      rgb_size[0] = 8;
      rgb_size[1] = 8;
      rgb_size[2] = 8;
      break;
    }

  SDL_GL_SetAttribute (SDL_GL_RED_SIZE, rgb_size[0]);
  SDL_GL_SetAttribute (SDL_GL_GREEN_SIZE, rgb_size[1]);
  SDL_GL_SetAttribute (SDL_GL_BLUE_SIZE, rgb_size[2]);
  SDL_GL_SetAttribute (SDL_GL_ALPHA_SIZE, 0);
  SDL_GL_SetAttribute (SDL_GL_DEPTH_SIZE, 24);
  SDL_GL_SetAttribute (SDL_GL_DOUBLEBUFFER, 1);

  if (fsaa)
    {
      SDL_GL_SetAttribute (SDL_GL_MULTISAMPLEBUFFERS, 1);
      SDL_GL_SetAttribute (SDL_GL_MULTISAMPLESAMPLES, fsaa);
    }

  if (SDL_SetVideoMode (w, h, bpp, video_flags) == NULL)
    {
      fprintf (stderr, "Couldn't set GL mode: %s\n", SDL_GetError ());
      SDL_Quit ();
      exit (1);
    }

  //printf ("Screen BPP: %d\n", SDL_GetVideoSurface ()->format->BitsPerPixel);
  //printf ("\n");
  //printf ("Vendor     : %s\n", glGetString (GL_VENDOR));
  //printf ("Renderer   : %s\n", glGetString (GL_RENDERER));
  //printf ("Version    : %s\n", glGetString (GL_VERSION));
  //printf ("Extensions : %s\n", glGetString (GL_EXTENSIONS));
  printf ("\n");

  SDL_GL_GetAttribute (SDL_GL_RED_SIZE, &value);
  printf ("SDL_GL_RED_SIZE: requested %d, got %d\n", rgb_size[0], value);
  SDL_GL_GetAttribute (SDL_GL_GREEN_SIZE, &value);
  printf ("SDL_GL_GREEN_SIZE: requested %d, got %d\n", rgb_size[1], value);
  SDL_GL_GetAttribute (SDL_GL_BLUE_SIZE, &value);
  printf ("SDL_GL_BLUE_SIZE: requested %d, got %d\n", rgb_size[2], value);
  SDL_GL_GetAttribute (SDL_GL_ALPHA_SIZE, &value);
  printf ("SDL_GL_ALPHA_SIZE: requested ?, got %d\n", value);
  SDL_GL_GetAttribute (SDL_GL_DEPTH_SIZE, &value);
  printf ("SDL_GL_DEPTH_SIZE: requested %d, got %d\n", bpp, value);
  SDL_GL_GetAttribute (SDL_GL_DOUBLEBUFFER, &value);
  printf ("SDL_GL_DOUBLEBUFFER: requested 1, got %d\n", value);

  if (fsaa)
    {
      SDL_GL_GetAttribute (SDL_GL_MULTISAMPLEBUFFERS, &value);
      printf ("SDL_GL_MULTISAMPLEBUFFERS: requested 1, got %d\n", value);
      SDL_GL_GetAttribute (SDL_GL_MULTISAMPLESAMPLES, &value);
      printf ("SDL_GL_MULTISAMPLESAMPLES: requested %d, got %d\n", fsaa,
	      value);
    }

  /* Set the window manager title bar */
  SDL_WM_SetCaption ("libgender rendering test", "gendertest");

  /* Set the gamma for the window */
  if (gamma != 0.0)
    SDL_SetGamma (gamma, gamma, gamma);

  testmat = new test_material;
  testmat2 = new mat_gouraud_shaded (colour (1, 1, 0));
  testmat3 = new mat_timed ();
  debugmat = new mat_debug ();

#if 1
  entity_moveable *planet = new entity_moveable (new geometry_sphere (testmat2, 10));
  planet->move (vec3 (0, 0, -20));
  planet->show ();

  planet->v = vec3 (10, 0, 0);
  moveables.push_back (planet);
#endif

  doom3parse ("test.proc");

  skybox sb (
      "textures/skybox_mystery/1.jpg",
      "textures/skybox_mystery/2.jpg",
      "textures/skybox_mystery/3.jpg",
      "textures/skybox_mystery/4.jpg",
      "textures/skybox_mystery/5.jpg",
      "textures/skybox_mystery/6.jpg"
  );
  world_skybox = &sb;

  for (int i = 0; i <= 0; i++)
    {
      // load a entity
      txtprt_parser p;
      geometry *g;
      try
        {
          g = p.read ("test.blasc");
        }
      catch (txtprt_i_exception & e)
        {
          cout << "ERR: " << e.msg << endl;
        }

      entity *e = new entity (g);
      e->move (vec3 (i*5, -3, -i*10));
      e->show ();
    }

#if 1
  {
    fisch (vec3 (0, 0, -2e9), 1e9, 8);
    //entity *planet = new entity (new geometry_sphere (1e9));
    //planet->move (vec3 (0, 0, -1.5e9));
    //planet->show ();
  }

  {
    entity *planet = new entity (new geometry_sphere (testmat3, 4e15));
    planet->move (vec3 (0, 0, 1e17));
    planet->show ();
  }

  {
    RandomBuilding r;
    entity *randlvl = r.draw (1000, 1000, 200);
    randlvl->move (vec3 (0, 0, 0));
    randlvl->show ();

    entity *randlvl2 = r.draw (1000, 10000, 1000);
    randlvl2->move (vec3 (-20000, 0, 0));
    randlvl2->show ();

    entity *randlvl3 = r.draw (100, 30, 3);
    randlvl3->move (vec3 (-1000, 0, 0));
    randlvl3->show ();


    entity *randlvl4 = r.draw (100, 1000000, 100000);
    randlvl4->move (vec3 (0, -1001000, 0));
    randlvl4->show ();
  }

  //draw_floor (10, -500, -10, -1000);
#endif
  draw_level ();

#if 0
  {
    geometry_heightfield *hf = new geometry_heightfield (100000., 100000.);
    entity *e = new entity (hf);
    e->move (vec3 (-100000, -100000 * 0.01, -100000));
    e->show ();
  }
#endif

  //draw_test_nurb ();

  //camera.orig.x = 108; camera.orig.y = 0; camera.orig.z = -368;
  camera.orig.x = 0; camera.orig.y = 0; camera.orig.z = 0;
  camera.p = point (0, 0, 10);
  camera.d = vec3 (0, 0, -1);
  camera.u = vec3 (0, 1, 0);
  camera.w = w; camera.h = h;
  camera.fov = 40.;

  glDisable (GL_ALPHA_TEST);

  /* Loop until done. */
  frames = 0;

  linear_light mylight2;//mylight, mylight2;

 // mylight.radius = 100000;
 // mylight.c = colour (1., 1, 1, 1.);
 // mylight.intensity = 1.F;

  mylight2.radius = 100000;
  mylight2.c = colour (0., 1., 1., 1.);
  mylight2.intensity = 1.F;
  mylight2.orig.x = 0; mylight2.orig.y = 0; mylight2.orig.z = 0;
  mylight2.p.x = 0; mylight2.p.y = 0; mylight2.p.z = 0;

  //pass_data pass_light (&mylight);
  pass_data pass_light2 (&mylight2);

  while (!done)
    {
      char *sdl_error;
      SDL_Event event;

      camera_angle  += 90 * camera_velocity_angle  * timer.diff;
      camera_angle2 += 90 * camera_velocity_angle2 * timer.diff;

      vec3 geradeaus = matrix::rotation (-camera_angle, vec3 (0, 1, 0)) * vec3 (0, 0, -1);
      vec3 right     = matrix::rotation (90., vec3 (0, 1, 0)) * geradeaus;

      camera.d = matrix::rotation (camera_angle2, right) * geradeaus;
      camera.u = cross (camera.d, right);

      camera.p = camera.p - camera.d * (camera_velocity_factor * timer.diff) * camera_velocity.z;
      camera.p = camera.p - camera.u * (camera_velocity_factor * timer.diff) * camera_velocity.y;

      //mylight.orig = camera.orig;
      //mylight.p = camera.p;

      mylight2.orig = camera.orig;
      //mylight2.p = vec3 (sin (timer.now / 2) * 100000, 1, cos (timer.now / 2) * 100000);

      //mylight.intensity = max (sinf (timer.now) + 1.2F, 0.2F);
      
      camera.begin ();
      camera.render (pass_depth);
      camera.render (pass_postdepth);
      camera.render (pass_light2);
      //camera.render (view::LIGHTED, pass_light2);
      camera.end ();

      perfom_moves ();

      SDL_GL_SwapBuffers ();
      timer.frame ();

#if 0
      /* Check for error conditions. */
      gl_error = glGetError ();

      if (gl_error != GL_NO_ERROR) fprintf (stderr, "testgl: OpenGL error: %d\n", gl_error);

      sdl_error = SDL_GetError ();

      if (sdl_error[0] != '\0')
	{
	  fprintf (stderr, "testgl: SDL error '%s'\n", sdl_error);
	  SDL_ClearError ();
	}

      /* Allow the user to see what's happening */
      //SDL_Delay (40);
#endif

      /* Check if there's a pending event. */
      while (SDL_PollEvent (&event))
	done = HandleEvent (&event);

      ++frames;

      //printf ("planet %p\n", planet);//D
    }

  if (global_image)
    {
      SDL_FreeSurface (global_image);
      global_image = NULL;
    }

  if (global_texture)
    {
      glDeleteTextures (1, &global_texture);
      global_texture = 0;
    }

  /* Destroy our GL context, etc. */
  SDL_Quit ();
  return (0);
}
int
main (int argc, char *argv[])
{
  int i, logo;
  int numtests;
  int bpp = 0;
  int slowly;
  float gamma = 0.0;
  int noframe = 0;
  int fsaa = 0;

  logo = 0;
  slowly = 0;
  numtests = 1;
  for (i = 1; argv[i]; ++i)
    {
      if (strcmp (argv[i], "-twice") == 0)
	{
	  ++numtests;
	}
      if (strcmp (argv[i], "-slow") == 0)
	{
	  slowly = 1;
	}
      if (strcmp (argv[i], "-bpp") == 0)
	{
	  bpp = atoi (argv[++i]);
	}
      if (strcmp (argv[i], "-gamma") == 0)
	{
	  gamma = (float) atof (argv[++i]);
	}
      if (strcmp (argv[i], "-noframe") == 0)
	{
	  noframe = 1;
	}
      if (strcmp (argv[i], "-fsaa") == 0)
	{
	  ++fsaa;
	}
      if (strncmp (argv[i], "-h", 2) == 0)
	{
	  printf
	    ("Usage: %s [-twice] [-logo] [-slow] [-bpp n] [-gamma n] [-noframe] [-fsaa]\n",
	     argv[0]);
	  exit (0);
	}
    }

  for (i = 0; i < numtests; ++i)
    RunGLTest (argc, argv, logo, slowly, bpp, gamma, noframe, fsaa);

  return 0;
}
